package com.qmth.boot.core.solar.service;

import com.qmth.boot.core.exception.StatusException;
import com.qmth.boot.core.solar.api.SolarApiClient;
import com.qmth.boot.core.solar.config.SolarProperties;
import com.qmth.boot.core.solar.crypto.AppLicenseParser;
import com.qmth.boot.core.solar.model.AppControl;
import com.qmth.boot.core.solar.model.AppInfo;
import com.qmth.boot.core.solar.model.AppLicense;
import com.qmth.boot.core.solar.model.OrgInfo;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import java.util.LinkedList;
import java.util.List;

@Component
public class SolarService {

    private static final Logger log = LoggerFactory.getLogger(SolarService.class);

    private SolarProperties solarProperties;

    private SolarApiClient solarApiClient;

    private AppLicense license;

    private AppInfo appInfo;

    public SolarService(SolarProperties solarProperties, SolarApiClient solarApiClient) {
        this.solarProperties = solarProperties;
        this.solarApiClient = solarApiClient;
        if (solarProperties.isCheckup()) {
            checkup();
        }
    }

    /**
     * 获取APP基本信息
     *
     * @return
     */
    public AppInfo getAppInfo() {
        return appInfo;
    }

    /**
     * 获取APP控制参数
     *
     * @return
     */
    public AppControl getAppControl() {
        return appInfo.getControl();
    }

    /**
     * 获取APP可以访问的机构集合，自动判断在线/离线激活两种模式
     *
     * @return
     */
    public List<OrgInfo> getOrgList() {
        if (license != null) {
            //offline
            return license.getOrgs();
        } else {
            //online
            List<OrgInfo> list = new LinkedList<>();
            int pageNumber = 1;
            int pageSize = 20;
            while (true) {
                List<OrgInfo> page = solarApiClient.listOrgInfo(pageNumber, pageSize);
                if (page == null || page.isEmpty()) {
                    break;
                }
                list.addAll(page);
                pageNumber++;
            }
            return list;
        }
    }

    private void checkup() {
        log.info("Solar checkup starting...");
        if (StringUtils.isNotBlank(solarProperties.getAccessKey()) && StringUtils
                .isNotBlank(solarProperties.getAccessSecret())) {
            appInfo = solarApiClient.getAppInfo();
            if (appInfo == null) {
                throw new StatusException("Solar online validate faile!");
            }
        } else if (StringUtils.isNotBlank(solarProperties.getLicense())) {
            license = AppLicenseParser.parse(solarProperties.getLicense());
            if (license == null) {
                throw new StatusException("Solar license invalid");
            }
            appInfo = license;
            if (getAppControl().hasExpired()) {
                throw new StatusException("Solar license has expired");
            }
        } else {
            throw new StatusException("Solar checkup faile, accessKey&accessSecret or license file is required!");
        }
        log.info("Solar checkup success, current app name: " + appInfo.getName());
    }

}
